/**************************************************************
*
*                FILTER.C
*
**************************************************************/

#include <acs.h>
#include <acsplus.h>
#include <string.h>   /* because of memcmp(), ... */
#include <stdlib.h>   /* because of atol(), ... */
#include <messages.pif>   /* for use of self-built library */
#include <notimtab.pif>   /* for use of self-built library */
#include <servimem.pif>   /* for use of self-built library */
#include <acs_plus.pif>   /* for use of self-built library */
#include <graphics.pif>   /* for use of self-built library */

#include "filter.pif"
#include "filter.h"
#include "filter.ah"


#define max_track_name_length 29
static char default_name[max_track_name_length] = "                    " ;


/************* OBJECT DATA PROTOTYPE BEGIN ***********************/

typedef struct 
{
	char name[max_track_name_length] ;
	NOTE_TIME_TABLE_ELEMENT *first_element ;
	long elements_generated ;
	NOTE min_note, max_note ;
	SWITCH track_filter ; 
	unsigned int channel_filter, channels ;
	int last ;
} 
TRACK_TABLE_ELEMENT ;


/*** GEM-object numbers for channel buttons 0..F ***/
static int object_numbers[16] = 
	{ _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _A, _B, _C, _D, _E, _F } ;

/************* OBJECT DATA PROTOTYPE END *************************/


/*******************************************************************
*/
	static void keys(Awindow *wi, int kstate, int key)
/*
*   Keyboard procedure of the filter window. See ACS manual.
*   The filter window contains editable fields. Therefore 
*   nothing is forwarded to other application windows.
*
*******************************************************************/
{
	/*** forward event to ACS for further treatment (e.g. by the desktop) ***/
	Awi_keys(wi, kstate, key) ;
}


/*******************************************************************
*/
	static void own_to_obj ( Awindow *wi, 
                         TRACK_TABLE_ELEMENT *current_track )
/*
* copies data of a track table element
* into the corresponding GEM-object. 
* This is done for the current track. The GEM-object
* only displays parameters of one track at a time.
* 
* INPUT:   window:          pointer to a current window
*          current_track:   pointer to current track table element
* OUTPUT:  refreshed GEM-object on screen 
* 
*******************************************************************/
{
TRACK_TABLE_ELEMENT *first_track = wi->user ;
int i ;
char *text ;

	/*** track name ***/
	text = wi->work[TRACK_NAME].ob_spec.tedinfo->te_ptext ;
	*text = '\0' ;
	strncat(text, current_track->name, max_track_name_length) ;

	/*** track_number ***/
	itoa( ((long)current_track - (long)first_track)/sizeof *first_track,
	     wi->work[TRACK].ob_spec.tedinfo->te_ptext, 10) ;
	/*** radio buttons ***/
	if (current_track->track_filter == on)
	{
		wi->work[ON].ob_state |= SELECTED ;
		wi->work[OFF].ob_state &= ~SELECTED ;
	}
	else 
	{
		wi->work[OFF].ob_state |= SELECTED ;
		wi->work[ON].ob_state &= ~SELECTED ;
	}
	/*** disable buttons if note time table is empty ***/
	if (current_track->first_element == NULL)
	{
		wi->work[ON].ob_state |= DISABLED ; 
		wi->work[OFF].ob_state |= DISABLED ;
	}
	else
	{
		wi->work[ON].ob_state &= ~DISABLED ; 
		wi->work[OFF].ob_state &= ~DISABLED ;
	}
	/*** channel filter ***/
	for (i = 0 ; i < 16 ; i++)
	{
		if (current_track->channel_filter  & (0x0001 << i) ) 
			wi->work[object_numbers[i]].ob_state |= SELECTED ;
		else wi->work[object_numbers[i]].ob_state &= ~SELECTED ;

		/*** disable buttons if channel or note time table is empty ***/
		if ( !(((current_track->channels)>>i) & 0x1)
		     || (current_track->first_element == NULL) )
			wi->work[object_numbers[i]].ob_state |= DISABLED ;
		else
			wi->work[object_numbers[i]].ob_state &= ~DISABLED ;
	} 
	/*** refresh ***/
	Awi_obchange(wi, ON, -1) ;
	Awi_obchange(wi, OFF, -1) ;
	Awi_obchange(wi, TRACK_CARRIER, -1) ;
	Awi_obchange(wi, NAME_CARRIER, -1) ;
	Awi_obchange(wi, CHANNELS, -1) ;		
}


/*******************************************************************
*/
	static TRACK_TABLE_ELEMENT *obj_to_own(Awindow *wi) 
/*
* copies parameters of a GEM-object
* which could have been manipulated by the program user
* into the corresponding element of a track table. 
* 
* RETURN:  pointer to track table element which corresponds
*          to the track shown in the GEM-object
* 
*******************************************************************/
{
TRACK_TABLE_ELEMENT *first_track = wi->user, *current_track ;
int i ;

	/*** track_number ***/
	current_track = first_track + atol(wi->work[TRACK].ob_spec.tedinfo->te_ptext) ;

	/*** radio buttons ***/
	if (wi->work[ON].ob_state & SELECTED)  current_track->track_filter = on ;
	else current_track->track_filter = off ;

	/*** channels buttons ***/
	for (i = 0 ; i < 16 ; i++)
	{
		if (wi->work[object_numbers[i]].ob_state & SELECTED) 
			current_track->channel_filter |= (0x0001 << i) ;
		else current_track->channel_filter &= ~(0x0001 << i) ;
	} 
	/*** return pointer referring to displayed GEM-object ***/
	return current_track ;
}


/*******************************************************************
*/
	static int service(Awindow *wi, int task, void *not_used)
/*
* Service routine for messages sent to this window.
* See also ACS manual.
* 
* OUTPUT:  AS_TERM ==> message is forwarded to parent object
*          AS_FILTER ==> a little help function 
* 
*******************************************************************/
{
	switch (task)
	{	
		case AS_TERM: Awi_sendall(TERMINATE, wi) ; break ; 
		case AS_INFO: A_dialog(&FILTER_INFO) ; break ;
		default:      return FALSE ;
	}
	return TRUE ;	/* task has been treated */
}


/*******************************************************************
*/
	static void button(void) 
/*
* Click routine for the FILTER object. 
* The buttons "next" and "previous" allow to select a track
* for displaying it in the work object. 
* 
* OUTPUT:  GEM-object data and own object data, modified
* 
*******************************************************************/
{
TRACK_TABLE_ELEMENT *first_track = ev_window->user, *current_track ;

	/*** save edited data of GEM-object ***/
	current_track = obj_to_own(ev_window) ;

	/*** determine new current track ***/
	switch (ev_obnr)
	{
	case PREVIOUS:
		if (current_track != first_track) current_track -= 1 ;
		own_to_obj(ev_window, current_track) ;
		break ;
	case NEXT:
		if (!current_track->last) current_track += 1 ;
		own_to_obj(ev_window, current_track) ;
		break ;
	default:
		/*** update protocol window ***/
		Awi_sendall(REDRAW, ev_window) ;
	}
}


/*******************************************************************
*
*              PUBLISHED INTERFACE 
*
*******************************************************************/

void FILTER_destructor(Awindow *wi)
{
TRACK_TABLE_ELEMENT *current_track ;

	if (wi) 
	{
		if (wi->user)
		{
			current_track = wi->user ;
			/*** free the note time tables ***/
			memory_symptom = "FiDstr NTTs" ;
			while (!current_track->last)
			{
				if (current_track->first_element != NULL)
					My_free(current_track->first_element) ;
				current_track += 1 ;
			}
			/*** also the very last ***/
			memory_symptom = "FiDstr last NTT" ;
			if (current_track->first_element != NULL)
				My_free(current_track->first_element) ; 
			/*** free track table ***/
			memory_symptom = "FiDstr wi->user" ;
			My_free(wi->user) ;
		}
		memory_symptom = "FiDstr wi" ;
		Awi_delete(wi) ;
	}
}


Awindow *FILTER_constructor(FILTER_INIT *filter_init)
{
Awindow *wi ;
TRACK_TABLE_ELEMENT *current_track ;
unsigned char *address ;
unsigned long length ;
long transformed_size ;
char MTrk[5] = "MTrk" ;

	/*** enter filename into icon text ***/
	FILTER_WINDOW.iconblk = filter_init->icon ;
	FILTER_WINDOW.iconblk->monoblk.ib_ptext = 
	FILTER_WINDOW.info = filter_init->filename ;

	/*** create the FILTER window ***/
	wi = Awi_create(&FILTER_WINDOW) ;
	if (!wi) return NULL ;

	/*** allocate track table ***/
	wi->user = current_track = My_alloc(filter_init->ntrks * sizeof *current_track) ;
	if (!current_track) { FILTER_destructor(wi) ; return NULL ; }

	/*** transform MIDI file ***/
	address = filter_init->RAM_file ;
	while (address < filter_init->RAM_file + filter_init->filesize)
	{
		/*** get the length of the next chunk ***/
		length = read_long(address + 4) ;
		if ( !memcmp(MTrk, address, 4) )
		{
			/*** This is a track chunk: ***/
			/*** init track data ***/
	 		current_track->first_element = NULL ;
			current_track->elements_generated = 0 ; 
			current_track->track_filter = filter_init->track_filter ; 
			current_track->channel_filter = filter_init->channel_filter ;
			strcpy(current_track->name, default_name) ;
			current_track->last = TRUE ;
			if (current_track != wi->user) (current_track - 1)->last = FALSE ;

			/*** allocate enough RAM for note time table ***/
			current_track->first_element 
			  = My_alloc( (length/3 + 1) * sizeof *current_track->first_element )  ;
			if (!current_track->first_element) { FILTER_destructor(wi) ; return NULL ; }

			/*** fill note time table ***/
			if ( fill_note_time_table(address + 4 + 4, length, current_track->first_element, 
			  &current_track->elements_generated, &current_track->channels,
			  &current_track->min_note, &current_track->max_note,
			  current_track->name, max_track_name_length) != note_time_table_filled ) 
			{
				alert_str(INVALID_FILE_FORMAT, filter_init->filename) ;
				FILTER_destructor(wi) ;
				return NULL ;
			}
			if ( !current_track->elements_generated ) current_track->track_filter = off ;

			/*** shrink RAM to right size ***/
			transformed_size = current_track->elements_generated * sizeof *(current_track->first_element) ;
			memory_symptom = "FiCstr shrink" ;
			current_track->first_element = My_shrink(current_track->first_element, transformed_size) ;

			/*** address next track table element ***/
			current_track += 1 ; 
		}
		/*** address next chunk ***/
		address = address + length + 4 + 4 ;	
	}
	/*** enter data in GEM-object ***/
	own_to_obj(wi, wi->user) ;

	/*** return success message ***/
	return wi ;
}


long get_transformed_size(Awindow *wi)
{
TRACK_TABLE_ELEMENT *current_track = wi->user ;
long total = current_track->elements_generated 
              * sizeof *(current_track->first_element) ;

	/*** sum up sizes of note time tables ***/
	while (!current_track->last)
	{
		current_track += 1 ;
		total += current_track->elements_generated 
		          * sizeof *(current_track->first_element) ;
	}
	return total ;
}


void get_absolute_min_max_note(Awindow *wi, 
                               NOTE *min_note, NOTE *max_note)
{
TRACK_TABLE_ELEMENT *current_track = wi->user ;
*min_note = *max_note = 0 ;

	/*** update own data ***/
	obj_to_own(wi) ;

	/*** search absolute minimum and maximum enabled notes of all enabled tracks ***/
	if (current_track->track_filter == on)
	{
		*min_note = current_track->min_note ;
		*max_note = current_track->max_note ;
	}
	while (!current_track->last)
	{
		current_track += 1 ;
		if (current_track->track_filter == on)
	 	{
			if (current_track->min_note < *min_note)
				*min_note = current_track->min_note ;
			if (current_track->max_note > *max_note)
				*max_note = current_track->max_note ;
		}
	}
} 


long get_absolute_max_time(Awindow *wi)
{
TRACK_TABLE_ELEMENT *current_track = wi->user ;
TIME time = 0 ;

	/*** update own data ***/
	obj_to_own(wi) ;

	/*** search the highest time of all events of all enabled tracks ***/
	if (current_track->track_filter == on)
		time = (current_track->first_element 
		        + current_track->elements_generated - 1)->time ;
	while (!current_track->last)
	{
		current_track += 1 ;
		if (current_track->track_filter == on)
		{
			if ((current_track->first_element 
			     + current_track->elements_generated - 1) ->time > time)
		 		time = (current_track->first_element 
				        + current_track->elements_generated - 1)->time ;
		}
	}
	return time ;
} 


int get_first_active_track(Awindow *wi) 
{
TRACK_TABLE_ELEMENT *first_track = wi->user ;
int track = 0 ;

	/*** update own data ***/
	obj_to_own(wi) ;

	/*** search the least track number which refers to an enabled track ***/
	if (first_track->track_filter == on) return track ;
	while (!(first_track + track)->last)
	{
		track ++ ;
		if ( (first_track + track)->track_filter == on ) return track ;
	}
	return -1 ;
}


int get_last_active_track(Awindow *wi) 
{
TRACK_TABLE_ELEMENT *first_track = wi->user ;
int track = 0, last_track = -1 ;

	/*** update own data ***/
	obj_to_own(wi) ;

	/*** search the highest track number which refers to an enabled track ***/
	if (first_track->track_filter == on) last_track = track ;
	while (!(first_track + track)->last)
	{
		track ++ ;
		if ( (first_track + track)->track_filter == on ) last_track = track ;
	}
	return last_track ;
}


char *get_track_name(Awindow *wi, int track) 
{ return ((TRACK_TABLE_ELEMENT *)(wi->user) + track)->name ; }


int get_min_max(Awindow *wi, int track, 
				TIME min_time , TIME max_time, 
            	NOTE *min_note, NOTE *max_note) 
{
TRACK_TABLE_ELEMENT *t = ((TRACK_TABLE_ELEMENT *)wi->user) + track ;
NOTE_TIME_TABLE_ELEMENT *n, *last_element = t->first_element + t->elements_generated - 1 ; 

	/*** update own data ***/
	obj_to_own(wi) ;

	/*** if track is disabled, return ***/
	if (t->track_filter == off) return 0 ;
	n = get_element(t->first_element, t->elements_generated, min_time) ;
	if ( (!n) ||                    /* no element beyond min_time found */
	     (n->time >= max_time) )    /* element found, but its time is too high */
	{
		*max_note = *min_note = 0x00 ;
		return 1 ;
	}
	/*** if max. time is higher than the highest time of the track reduce it ***/
	if ( last_element->time < max_time )
		max_time = (t->first_element + t->elements_generated - 1)->time ;
	/*** search for minimum and maximum ***/
	*min_note = *max_note = n->note & ~_ON_ ;
	while ( (n->time <= max_time) && (n <= last_element) )
	{
		if ( t->channel_filter & (0x1 << n->channel) )
		{
			if ((n->note & ~_ON_) > *max_note) 
				*max_note = n->note & ~_ON_ ;
			if ((n->note & ~_ON_) < *min_note) 
				*min_note = n->note & ~_ON_ ;
		}
		n += 1 ;
	}
	return 1 ;
}


void draw_track
(
	int handle,
	Awindow *wi, 
	int track,
	int x_min, int x_max, int y, float dy,
	TIME min_time, TIME max_time, NOTE note_of_y,
	int average_height, float dyn_scale,
	int type, int style[12], int color[12]
)
{
TRACK_TABLE_ELEMENT *t = ((TRACK_TABLE_ELEMENT *)wi->user) + track ;
NOTE_TIME_TABLE_ELEMENT *n ;
float time_scale = ((float)(x_max - x_min)) / (max_time - min_time) ;
int points[4] ;
int y_note, half_height, dynamic ;
int original_type = type, index /* for style and color arrays */ ;

	/*** update own data ***/
	obj_to_own(wi) ;

	/*** draw all notes between minimum and maximum time ***/
	n = get_element(t->first_element, t->elements_generated, min_time) ;
	if (!n) return ;
	while ( (n < t->first_element + t->elements_generated) && 
	        (n->time <= max_time) ) 
	{
		if ( (t->track_filter) && (t->channel_filter & (1 << n->channel)) )
		{
			points[0] = x_min + (int)(time_scale * (n->time - min_time)) ;
			points[2] = x_min + (int)(time_scale 
			                          * ((n + n->reference)->time - min_time)) ;
			/*** if note is wider than the useable x-range: cut ***/
			if ( points[2] > x_max ) 
			{ 
				points[2] = x_max ; 
				type |= TAIL_CUT ;
			}
			if ( points[2] < x_min ) 
			{
				points[2] = points[0] ; 
				points[0] = x_min ; 
				type |= HEAD_CUT ; 
			}
			/*** if points[2] corresponds to a "note on" event or a "note off" ***/
			/*** event with its "note on" in a previous time range: draw note  ***/
			if ( points[2] > points[0] )
			{
				y_note = (int)(dy * ((n->note & ~_ON_) - note_of_y)) + y ;
				if (n->note & _ON_) dynamic = n->dynamic ; 
				else dynamic = (n + n->reference)->dynamic ;  
				half_height =  (int)( ( dyn_scale * (dynamic - 63) / 64 + 1 ) *
				                      average_height / 2 ) ;
				points[1] = y_note - half_height ;
				points[3] = y_note + half_height ;

				/*** draw note ***/
				index = (n->note & ~_ON_)%12 ;
				draw_note(handle, points, type, style[index], color[index]) ;
			}
		}
		n += 1 ; type = original_type ;
	}
}